"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var howler_1 = require("howler");
var bpm_1 = __importDefault(require("../visualization/bpm"));
var circular_wave_1 = require("../visualization/circular-wave");
var Playlist = /** @class */ (function () {
    function Playlist(sources, options) {
        var _this = this;
        this.currentIndex = 0;
        this.currentSoundId = 0;
        this.fadeInInMs = 1000;
        this.fadeOutInMs = 1200;
        this.options = options;
        this.sounds = sources.map(function (s) { return _this.createSound(s); });
        if (this.options.withVisualization) {
            this.circularWave = new circular_wave_1.CircularAudioWave(howler_1.Howler.ctx, howler_1.Howler.masterGain, options.colors || {});
        }
    }
    Playlist.prototype.createSound = function (src) {
        var _this = this;
        var playable = new howler_1.Howl({
            src: [src],
            onend: function () {
                // jump to the next sound after the first if there is one
                if (_this.currentIndex < _this.sounds.length - 1) {
                    _this.currentIndex++;
                }
                else {
                    _this.currentIndex = 0;
                }
                // do not continue to play the next sound
                // instead pause the playlist here
                if (_this.options.oneSoundPerRequest) {
                    if (_this.circularWave) {
                        _this.circularWave.stop();
                    }
                    return;
                }
                setTimeout(function () {
                    _this.play();
                }, 500);
            }
        });
        var result = { playable: playable };
        if (this.options.withVisualization) {
            // calculate the bpm for the waves ripple effect
            bpm_1.default(src).then(function (bpm) { return (result.bpm = bpm); });
        }
        return result;
    };
    Playlist.prototype.play = function () {
        var _a = this.sounds[this.currentIndex], playable = _a.playable, bpm = _a.bpm;
        if (this.circularWave) {
            this.circularWave.start(bpm || 120);
        }
        playable.fade(0, 1, this.fadeInInMs);
        playable.play();
    };
    Playlist.prototype.pause = function () {
        var playable = this.sounds[this.currentIndex].playable;
        // ignore pause since we wait till the sound ended
        if (this.options.oneSoundPerRequest) {
            return;
        }
        if (this.circularWave) {
            this.circularWave.stop();
        }
        // do not fade and pause when not playing to not cause sideeffects
        // but calling after stopping the wave since that could be
        // called during two sounds, with the wave still active
        if (!playable.playing()) {
            return;
        }
        playable.fade(1, 0, this.fadeOutInMs);
        setTimeout(function () {
            playable.pause();
        }, this.fadeOutInMs);
    };
    return Playlist;
}());
exports.Playlist = Playlist;
//# sourceMappingURL=playlist.js.map