"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var __1 = require("..");
var OfflineAudioContext = window.OfflineAudioContext ||
    window.webkitOfflineAudioContext;
var offlineContext;
var offlineSource;
if (!OfflineAudioContext) {
    console.warn('AUDIO LOGGER VISUALIZATION: no audio web api support');
}
else {
    offlineContext = new OfflineAudioContext(2, 30 * 44100, 44100);
    offlineSource = offlineContext.createBufferSource();
    // Beats generally occur around the 100 to 150 hz range.
    var lowpass = offlineContext.createBiquadFilter();
    lowpass.type = 'lowpass';
    lowpass.frequency.setValueAtTime(150, 0);
    lowpass.Q.setValueAtTime(1, 0);
    offlineSource.connect(lowpass);
    var highpass = offlineContext.createBiquadFilter();
    highpass.type = 'highpass';
    highpass.frequency.setValueAtTime(100, 0);
    highpass.Q.setValueAtTime(1, 0);
    lowpass.connect(highpass);
    highpass.connect(offlineContext.destination);
}
function _getBPM(data) {
    var partSize = 22050, parts = data[0].length / partSize, peaks = [];
    for (var i = 0; i < parts; i++) {
        var max = 0;
        for (var j = i * partSize; j < (i + 1) * partSize; j++) {
            var volume = Math.max(Math.abs(data[0][j]), Math.abs(data[1][j]));
            if (!max || volume > max.volume) {
                max = {
                    position: j,
                    volume: volume
                };
            }
        }
        peaks.push(max);
    }
    peaks.sort(function (a, b) {
        return b.volume - a.volume;
    });
    peaks = peaks.splice(0, peaks.length * 0.5);
    peaks.sort(function (a, b) {
        return a.position - b.position;
    });
    var groups = [];
    peaks.forEach(function (peak, index) {
        var _loop_1 = function (i) {
            var group = {
                bpm: (60 * 44100) / (peaks[index + i].position - peak.position),
                count: 1
            };
            while (group.bpm < 90) {
                group.bpm *= 2;
            }
            while (group.bpm > 180) {
                group.bpm /= 2;
            }
            group.bpm = Math.round(group.bpm);
            if (!groups.some(function (interval) {
                return interval.bpm === group.bpm ? interval.count++ : 0;
            })) {
                groups.push(group);
            }
        };
        for (var i = 1; index + i < peaks.length && i < 10; i++) {
            _loop_1(i);
        }
    });
    var bpm = groups.sort(function (intA, intB) {
        return intB.count - intA.count;
    })[0].bpm;
    return bpm;
}
exports.default = (function (filePath) {
    if (!OfflineAudioContext) {
        return Promise.reject('no audio context');
    }
    var request = new XMLHttpRequest();
    request.open('GET', filePath, true);
    request.setRequestHeader(__1.DISABLE_HEADER, 'yoman');
    request.responseType = 'arraybuffer';
    request.send();
    return new Promise(function (resolve, reject) {
        request.onload = function () {
            // Preprocess buffer for bpm
            offlineContext.decodeAudioData(request.response, function (buffer) {
                offlineSource.buffer = buffer;
                offlineSource.start(0);
                offlineContext.startRendering();
            });
            offlineContext.oncomplete = function (e) {
                var buffer = e.renderedBuffer;
                var bpm = _getBPM([
                    buffer.getChannelData(0),
                    buffer.getChannelData(1)
                ]);
                resolve(bpm);
            };
        };
        request.onerror = function () { return reject(); };
    });
});
//# sourceMappingURL=bpm.js.map